@@ -32,9 +32,6 @@ except ImportError:
 log = logging.getLogger(__name__)
 log.addHandler(NullHandler())
 
-# Cache the github issues to reduce duplicate lookups
-_issue_cache = {}
-
 # Maintain a list of github labels to consider issues "finished".  Any issues
 # associated with these labels will be considered "done".
 GITHUB_COMPLETED_LABELS = []
@@ -55,10 +52,10 @@ def pytest_addoption(parser):
                     default=None,
                     metavar='GITHUB_USERNAME',
                     help='GitHub username (defaults to value supplied in GITHUB_CFG)')
-    group.addoption('--github-api-token',
+    group.addoption('--github-token',
                     action='store',
-                    dest='github_api_token',
-                    metavar='GITHUB_API_TOKEN',
+                    dest='github_token',
+                    metavar='GITHUB_TOKEN',
                     default=None,
                     help='GitHub Personal Access token (defaults to value ' +
                     'supplied in GITHUB_CFG). Refer to ' +
@@ -90,46 +87,38 @@ def pytest_configure(config):
     # Sanitize key and token
     github_cfg_file = config.getoption('github_cfg_file')
     github_username = config.getoption('github_username')
-    github_api_token = config.getoption('github_api_token')
+    github_token = config.getoption('github_token')
     github_completed = config.getoption('github_completed')
 
-    # If not --help or --collectonly or --showfixtures ...
+    # If not --help or --showfixtures ...
     if not (config.option.help or config.option.showfixtures):
         # Warn if file does not exist
         if not os.path.isfile(github_cfg_file):
             errstr = "No github configuration file found matching: %s" % github_cfg_file
             log.warning(errstr)
             warnings.warn(errstr, Warning)
-
-        # Load configuration file ...
-        if os.path.isfile(github_cfg_file):
+        else:
+            # Load configuration file ...
             with open(github_cfg_file, 'r') as fd:
                 github_cfg = yaml.load(fd)
                 try:
                     github_cfg = github_cfg.get('github', {})
-                except AttributeError:
+                except AttributeError as e:
                     github_cfg = {}
-                    errstr = "No github configuration found in file: %s" % github_cfg_file
+                    errstr = "No github configuration found in file: %s (%s)" % (os.path.realpath(github_cfg_file), e)
                     log.warning(errstr)
                     warnings.warn(errstr, Warning)
 
             if github_username is None:
-                github_username = github_cfg.get('key', None)
-            if github_api_token is None:
-                github_api_token = github_cfg.get('token', None)
+                github_username = github_cfg.get('username', None)
+            if github_token is None:
+                github_token = github_cfg.get('token', None)
             if github_completed is None or github_completed == []:
                 github_completed = github_cfg.get('completed', [])
 
-        # Initialize github api connection
-        api = github3.login(github_username, github_api_token)
-
-        # If completed is still empty, load default ...
-        if github_completed is None or github_completed == []:
-            github_completed = GITHUB_COMPLETED_LABELS
-
         # Register pytest plugin
         assert config.pluginmanager.register(
-            GitHubPytestPlugin(api, completed_labels=github_completed),
+            GitHubPytestPlugin(github_username, github_token, completed_labels=github_completed),
             'github_helper'
         )
 
@@ -175,18 +164,33 @@ class GitHubPytestPlugin(object):
 
     """GitHub Plugin class."""
 
-    def __init__(self, api, **kwargs):
+    def __init__(self, username, password, completed_labels=[]):
         """Initialize attributes."""
         log.debug("GitHubPytestPlugin initialized")
-        self.api = api
-        self.completed_labels = kwargs.get('completed_labels', [])
+
+        # initialize issue cache
+        self._issue_cache = {}
+
+        # Process parameters
+        self.username = username
+        self.password = password
+        # Initialize completed_labels
+        if completed_labels is None or completed_labels == []:
+            self.completed_labels = GITHUB_COMPLETED_LABELS
+        else:
+            self.completed_labels = completed_labels
+
+        # Initialize github api connection
+        self.api = github3.login(self.username, self.password)
 
     def __parse_issue_url(self, url):
         # Parse the github URL
         match = re.match(r'https?://github.com/([^/]+)/([^/]+)/(?:issues|pull)/([0-9]+)$', url)
-        if match is None:
-            raise Exception("Unhandled github issue URL: %s" % url)
-        return match.groups()
+        try:
+            return match.groups()
+        except AttributeError:
+            errstr = "Malformed github issue URL: '%s'" % url
+            raise Exception(errstr)
 
     def __cache_github_issues(self, items):
         """Collect github markers and populate the issue_cache."""
@@ -194,14 +198,18 @@ class GitHubPytestPlugin(object):
             marker = item.get_marker('github')
             issue_urls = tuple(sorted(set(marker.args)))  # (O_O) for caching
             for issue_url in issue_urls:
-                if issue_url not in _issue_cache:
+                # Handle missing/bogus args
+                if issue_url is None:
+                    continue
+                # Parse github issue
+                if issue_url not in self._issue_cache:
                     # parse the URL
                     (username, repository, number) = self.__parse_issue_url(issue_url)
 
                     try:
-                        _issue_cache[issue_url] = self.api.issue(username, repository, number)
-                    except AttributeError:
-                        errstr = "Unable to inspect github issue %s" % issue_url
+                        self._issue_cache[issue_url] = self.api.issue(username, repository, number)
+                    except (AttributeError, github3.models.GitHubError) as e:
+                        errstr = "Unable to inspect github issue %s - %s" % (issue_url, str(e))
                         warnings.warn(errstr, Warning)
             item.funcargs["github_issues"] = issue_urls
 
@@ -214,13 +222,13 @@ class GitHubPytestPlugin(object):
         unresolved_issues = []
         issue_urls = item.funcargs["github_issues"]
         for issue_url in issue_urls:
-            if issue_url not in _issue_cache:
+            if issue_url not in self._issue_cache:
                 continue
                 # warnings.warn(errstr, Warning)
 
-            issue = _issue_cache[issue_url]
+            issue = self._issue_cache[issue_url]
 
-            issue_labels = [l.name for l in issue.labels()]
+            issue_labels = [l.name for l in issue.labels]
 
             # if the issue is open and isn't considered "completed" by any of the issue labels ...
             if not issue.is_closed() and not set(self.completed_labels).intersection(issue_labels):
@@ -230,21 +238,22 @@ class GitHubPytestPlugin(object):
         if unresolved_issues:
             # TODO - Add support for skip vs xfail
             skip = item.get_marker('github').kwargs.get('skip', False)
-            if False and skip:
+            if skip:
                 pytest.skip("Skipping due to unresolved github issues:\n{0}".format(
-                    "\n ".join(["{0} [{1}] {2}".format(i.url, i.state, i.title) for i in unresolved_issues])))
+                    "\n ".join(["{0} [{1}] {2}".format(i.html_url, i.state, i.title) for i in unresolved_issues])))
             else:
                 item.add_marker(pytest.mark.xfail(
                     reason="Xfailing due to unresolved github issues: \n{0}".format(
-                        "\n ".join(["{0} [{1}] {2}".format(i.url, i.state, i.title) for i in unresolved_issues]))))
+                        "\n ".join(["{0} [{1}] {2}".format(i.html_url, i.state, i.title) for i in unresolved_issues]))))
 
     def pytest_collection_modifyitems(self, session, config, items):
         """Report number of github issues collected."""
         log.debug("pytest_collection_modifyitems() called")
+
         reporter = config.pluginmanager.getplugin("terminalreporter")
         reporter.write("collected", bold=True)
 
         # cache the issues
         self.__cache_github_issues(items)
 
-        reporter.write_line(" {0} github issues".format(len(_issue_cache)), bold=True)
+        reporter.write_line(" {0} github issues".format(len(self._issue_cache)), bold=True)
